@@ -41,7 +41,7 @@ from .site import (
     MinioFileUploader, MinioFileDownloader,
     RequestRevisions, MultiRequestRevisions,
 )
-from .file import TrackedFile, Revision, FileManager
+from .file import TrackedFile, Revision, FileManager, DefaultFileMap
 from .kitsu import KitsuAPIWrapper, KitsuBindings
 from .dependency import DependencyTemplates
 
@@ -399,18 +399,6 @@ class KitsuConfig(flow.Object):
             )
 
 
-class DefaultFiles(flow.Object):
-
-    file_names = flow.Param('')
-
-
-class DefaultFilesMap(flow.Map):
-
-    @classmethod
-    def mapped_type(cls):
-        return DefaultFiles
-
-
 class ProjectSettings(flow.Object):
 
     ICON = ('icons.gui', 'settings')
@@ -420,7 +408,7 @@ class ProjectSettings(flow.Object):
     support = flow.Param("")
 
     settings = flow.Child(ContextualView)
-    default_files = flow.Child(DefaultFilesMap)
+    default_files = flow.Child(DefaultFileMap)
     non_editable_files = flow.Param("")
     auto_upload = flow.Param('').ui(
         tooltip='Enable <b>upload after publish</b> by default for all files matching the provided patterns')
@@ -791,6 +779,12 @@ class Project(flow.Object):
             self.project_settings_folder(),
             self.get_user_name()
         )
+    
+    def get_default_file_presets(self):
+        '''
+        Must return an instance of libreflow.baseflow.file.DefaultFileMap.
+        '''
+        return self.admin.project_settings.default_files
 
     def get_project_thumbnail2(self):
         image = self.admin.project_settings.project_thumbnail.get()
