@@ -72,7 +72,7 @@ class ResetJob(flow.Action):
     
     def get_buttons(self):
         self.message.set((
-            '<h3>Revision already available on exchange server</h3>'
+            '<h3>Revision already on the exchange server</h3>'
             'Reset upload job anyway ?'
         ))
         return ['Confirm', 'Cancel']
@@ -83,6 +83,7 @@ class ResetJob(flow.Action):
         
         self._job.status.set('WAITING')
         self._job.log.set('?')
+        self._job.on_submitted()
         self._jobs.incr_waiting_count()
         o = self.root().get_object(self._job.emitter_oid.get())
         o.set_sync_status('NotAvailable', exchange=True)
@@ -129,8 +130,7 @@ class Job(Entity):
     status              = Property(property_value_type=JobStatus)
     priority            = Property().ui(editor='int')
     emitter_oid         = Property()
-    local_path          = Property()
-    server_path         = Property()
+    path                = Property()
     date                = Property().ui(editor='datetime')
     date_end            = Property().ui(editor='datetime')
     is_archived         = Property().ui(editor='bool')
@@ -157,27 +157,82 @@ class Job(Entity):
 
         return job_repr
     
-    def on_submitted(self):
+    def get_local_path(self):
         '''
-        Computes the local and server paths of the job's
-        requested object (identified by `emitter_oid`).
+        Returns the local path of the associated object.
+        
+        Note that the result depends on the current site
+        since it includes the root folder of the latter.
+        '''
+        return os.path.join(
+            self.root().project().get_root(),
+            self.path.get()
+        )
+    
+    def get_server_path(self):
+        '''
+        Returns the path of the associated object on the server.
+        '''
+        path = self.path.get()
 
-        If the oid does not refer to an existing revision
-        object, the methods puts the job in error state.
-        '''
+        if os.path.splitext(path)[1] == '':
+            path += '/' + os.path.basename(path) + '.zip'
+        
+        return path
+    
+    def get_related_object(self):
         try:
-            revision = self.root().get_object(
+            o = self.root().get_object(
                 self.emitter_oid.get()
             )
         except (ValueError, flow.exceptions.MissingRelationError):
+            o = None
+        
+        return o
+    
+    def check_valid_state(self):
+        '''
+        Returns a boolean meaning whether this job is ready
+        to be processed or not.
+        
+        The method may update the job's properties to indicate
+        why this job cannot yet be processed. It assumes that
+        `emitter_oid` references a valid revision object.
+        '''
+        if self.status.get() != 'WAITING':
+            return False
+        
+        o = self.get_related_object()
+
+        if self.type.get() == 'Download' and o.get_sync_status(exchange=True) != 'Available':
+            self.log.set((
+                'Revision is not yet available '
+                'on the exchange server'
+            ))
+            return False
+        
+        return True
+    
+    def on_submitted(self):
+        '''
+        Method which may be called right after this job
+        has been submitted to a queue.
+
+        Its default behaviour is to compute the relative path
+        of the job's requested revision (`emitter_oid`).
+        If the oid does not refer to an existing revision
+        object, the methods puts the job in error state.
+        '''
+        revision = self.get_related_object()
+        
+        if revision is None:
             self.status.set('ERROR')
             self.log.set('Object does not exist')
             return
         
         try:
             ready_for_sync = revision.ready_for_sync.get()
-            local_path = revision.get_path()
-            server_path = revision.get_path(relative=True)
+            path = revision.get_path(relative=True).replace('\\', '/')
         except AttributeError:
             self.status.set('ERROR')
             self.log.set('Object is not a revision')
@@ -188,21 +243,16 @@ class Job(Entity):
                 self.log.set('Revision is not ready to be synced')
                 return
         
-        if os.path.splitext(server_path)[1] == '':
-            server_path += '/' + os.path.basename(server_path) + '.zip'
-        
-        self.local_path.set(local_path.replace('\\', '/'))
-        self.server_path.set(server_path.replace('\\', '/'))
+        self.path.set(path)
     
     def on_processed(self):
         '''
-        Method which may be called after this job has been
-        processed.
+        Method which may be called right after this job
+        has been processed.
 
-        Default is to update the related revision's statutes,
-        and update its UI and that of its parent map. The
-        method assumes param `emitter_oid` references a valid
-        revision object.
+        Its default behaviour is to update the related
+        revision statutes. The method assumes that
+        `emitter_oid` references a valid revision object.
         '''
         if self.status.get() != 'PROCESSED':
             return
@@ -548,13 +598,7 @@ class Synchronize(ProcessJobs):
         )
 
     def process(self, job):
-        if job.status.get() != 'WAITING':
-            self.root().session().log_warning(
-                'Job %s in %s status: skipping' % (
-                    job.name(),
-                    job.status.get()
-                )
-            )
+        if not job.check_valid_state():
             return
         
         try:
@@ -562,13 +606,13 @@ class Synchronize(ProcessJobs):
         
             if job.type.get() == 'Upload':
                 sync_manager.upload(
-                    job.server_path.get(),
-                    job.local_path.get()
+                    job.get_server_path(),
+                    job.get_local_path()
                 )
             else:
                 sync_manager.download(
-                    job.server_path.get(),
-                    job.local_path.get()
+                    job.get_server_path(),
+                    job.get_local_path()
                 )
         except Exception as e:
             job.status.set('ERROR')
@@ -758,7 +802,7 @@ class RequestAs(flow.Action):
 
         if revision_status == "Available" and not self.forced_upload.get():
             self.root().session().log_warning(
-                "Revision already available on exchange server"
+                "Revision already on the exchange server"
             )
             self.root().project().synchronization.touch()
             return self.get_result()
@@ -794,6 +838,8 @@ class RequestAs(flow.Action):
 
 class Request(RequestAs):
 
+    ICON = ('icons.libreflow', 'request')
+
     _revision = flow.Parent()
     priority = flow.IntParam(50)
     source_site = flow.Param(None, ActiveSiteRevisionAvailableChoiceValue)
@@ -1901,6 +1947,8 @@ class MultiRequestRevisions(RequestRevisions):
 
 
 class UploadRevision(flow.Action):
+
+    ICON = ('icons.libreflow', 'upload')
     
     _revision = flow.Parent()
     _revisions = flow.Parent(2)
@@ -1917,8 +1965,8 @@ class UploadRevision(flow.Action):
     
     def get_buttons(self):  
         self.message.set((
-            '<h3>Revision already available on exchange server</h3>'
-            'Upload and overwrite it anyway ?'
+            '<h3>Revision already on the exchange server</h3>'
+            'Upload and overwrite it on the server anyway ?'
         ))
         return ['Confirm', 'Cancel']
     
@@ -1942,15 +1990,28 @@ class UploadRevision(flow.Action):
 
 
 class DownloadRevision(flow.Action):
+
+    ICON = ('icons.libreflow', 'download')
     
     _revision = flow.Parent()
     _revisions = flow.Parent(2)
 
     def needs_dialog(self):
-        return False
+        return self._revision.get_sync_status() == 'Available'
+    
+    def get_buttons(self):  
+        self.message.set((
+            '<h3>Revision already available</h3>'
+            'Download and overwrite it locally anyway ?'
+        ))
+        return ['Confirm', 'Cancel']
     
     def allow_context(self, context):
-        return context and self._revision.get_sync_status() != 'Available'
+        return (
+            context
+            and not self._revision.is_working_copy()
+            and self._revision.get_sync_status(exchange=True) == 'Available'
+        )
     
     def run(self, button):
         if button == 'Cancel':
